library blusalt_identity;

import 'blusalt_sdk.dart';
import 'common/environment.dart';
import 'model/request/bvn_request.dart';
import 'model/request/dl_request.dart';
import 'model/request/ibvn_request.dart';
import 'model/request/nin_request.dart';
import 'model/request/nip_request.dart';
import 'model/request/pvc_request.dart';
import 'network/network_client.dart';
import 'package:blusalt_identity_verification/verification_manager.dart' as manager;

/// Initializing the SDK with API credentials obtained from account
/// [dashboard](blusalt.net). Replace the empty strings with the appropriate value
///
void initializeSDK() {
  ApiCredentials credentials =
  ApiCredentials(apiKey: "44JVFVI3PBIA6CCJVM7F62B2HM14SN3J", clientID: "NNVLI4IJOOUL", appName: "test-sersvices");
  BlusaltSDK.initializeSDK(credentials, env: Environment.test_debug);
}

/// Verify Customer BVN. Provide actual customer BVN and phone number
/// Get the Customer BVN data
void verifyBVN() async {
  try {
    BVNRequest bvnRequest = BVNRequest(bvnNumber: "22169344973");
    var response = await manager.verifyBVN(bvnRequest);
    print(response.results!.bvnNumber);
  }
  catch(e){
    print(e);
  }
}

/// Verify Customer Natianal Identity number. Provide actual customer NIN and phone number
/// Get the Customer NIN data
void verifyNIN() async {
  try {
    NINRequest ninRequest = NINRequest(ninNumber: "22169344973", phoneNumber: "");
    var response = await manager.verifyNIN(ninRequest);
    print(response.results!.documentNo);
  }
  catch(e){
    print(e);
  }
}

/// Verify Customer BVN Identity number. Provide actual customer BVN and phone number
/// Get the Customer BVN data including image
void verifyBVNWithImage() async {
  try {
    IBVNRequest bvnRequest = IBVNRequest(bvnNumber: "12345678");
    var response = await manager.verifyBVNWithImage(bvnRequest);
    print(response.results!.personalInfo);
  }
  catch(e){
    print(e);
  }
}

/// Verify Customer PVC. Provide actual customer PVC, phone number and state
/// Get the Customer PVC data
void verifyPVC() async {
  try{
  PVCRequest pvcRequest = PVCRequest(pvcNumber: "", phoneNumber: "",state: "", lastName: "");
  var response = await manager.verifyPVC(pvcRequest);
  print(response.results!.personalInfo);

  }
  catch(e){
    print(e);
  }
}


/// Verify Customer International passport. Provide actual customer Passport number and phone number
/// Get the Customer International passport data
void verifyNIP() async {
  try{
  var nipRequest = NIPRequest(passportNumber: "", phoneNumber: "", dateOfBirth: "", firstName: "", lastName: "");
  var response = await manager.verifyNIP(nipRequest);
  print(response.results.personalInfo);
  }
  catch(e){
    print(e);
  }

}

/// Verify Customer Driver's Licence. Provide actual customer licence and phone number
/// Get the Customer Driver's Licence  data
void verifyDriversLicence() async {
  try{
  var dlRequest = DLRequest(dateOfBirth: "", phoneNumber: "", licenceNumber: "", fullName: "");
  var response = await manager.verifyDriversLicence(dlRequest);
  print(response.results.personalInfo);
  }
  catch(e){
    print(e);
  }
}