import 'dart:collection';
import 'dart:io';


import 'package:blusalt_identity_verification/blusalt_sdk.dart';
import 'package:blusalt_identity_verification/common/environment.dart';
import 'package:http/http.dart' as http;
import 'package:http_interceptor/http_interceptor.dart';

class HttpClient {

  var _client = HttpClientWithInterceptor.build(interceptors: [
    LogginInterceptor(),
  ]);

  http.Client getClient(){
    return BlusaltSDK.environment == Environment.test_debug? _client : http.Client();
  }

  Future<http.Response> _makeGetRequest(
      String url, ApiCredentials credentials, body) {
    HashMap<String, String> headers =
        _createHeadersFromCredentials(credentials);
    return getClient().get(Uri.parse(url), headers: headers);
  }

  Future<http.Response> _makePostRequest(
      String url, ApiCredentials credentials, dynamic body) {
    HashMap<String, String> headers =
        _createHeadersFromCredentials(credentials);
    final link = BlusaltSDK.baseUrl + url;
    return getClient().post(Uri.parse(link), headers: headers, body: body);
  }

  Future<http.StreamedResponse> _makeMultiPartPostRequest(
      String url, ApiCredentials credentials, String file1, String file2) {
    HashMap<String, String> headers =
    _createHeadersFromCredentials(credentials);
    final link = BlusaltSDK.baseUrl + url;
    var file = http.MultipartFile('image_url_1', File(file1).readAsBytes().asStream(),
        File(file1).lengthSync(),
        filename: file1.split("/").last);
    var file_2 = http.MultipartFile('image_url_2', File(file2).readAsBytes().asStream(),
        File(file2).lengthSync(),
        filename: file2.split("/").last);
    var request = http.MultipartRequest('POST', Uri.parse(link));
    request.headers.addAll(headers);
    request.files.add(file);
    request.files.add(file_2);
    return request.send();
  }

  HashMap<String, String> _createHeadersFromCredentials(
      ApiCredentials credentials) {
    var headers = new HashMap<String, String>();
    headers["clientid"] = credentials.clientID;
    headers["appname"] = credentials.appName;
    headers["apikey"] = credentials.apiKey;
    headers["Content-Type"] = "application/json";
    return headers;
  }

  Future<dynamic> executeRequest(Request request) async{
    switch (request.requestMethod) {
      case Method.GET:
        return _makeGetRequest(
            request.url, request.apiCredentials, request.body);
      case Method.HEAD:
        // TODO: Handle this case.
        break;
      case Method.POST:
        return _makePostRequest(
            request.url, request.apiCredentials, request.body);
      case Method.PUT:
        // TODO: Handle this case.
        break;
      case Method.PATCH:
        // TODO: Handle this case.
        break;
      case Method.DELETE:
        // TODO: Handle this case.
        break;
    }
    return _makeGetRequest(
        request.url, request.apiCredentials, request.body);
  }

  Future<dynamic> executeMultipartRequest(MultiPartRequest request) async {
    return _makeMultiPartPostRequest(request.url, request.apiCredentials, request.filenames[0], request.filenames[1]);
  }

  Future<http.Response> initializeSDK(ApiCredentials credentials) {
    String url = BlusaltSDK.baseUrl  + "App/InitializeApp";
    return _makeGetRequest(url, credentials, null);
  }
}

class ApiCredentials {
  String apiKey, clientID, appName;
  ApiCredentials({required this.apiKey, required this.clientID, required this.appName});
}

class Request {
  late dynamic url, body;
  late ApiCredentials apiCredentials;
  late Method requestMethod;

  Request({this.url, this.body, required this.apiCredentials, required this.requestMethod});
}

class MultiPartRequest {
  late dynamic url;
  late ApiCredentials apiCredentials;
  late Method requestMethod;
  late List<String> filenames;

  MultiPartRequest({this.url, required this.apiCredentials, required this.requestMethod, required this.filenames });
}

class LogginInterceptor implements InterceptorContract {
  @override
  Future<RequestData> interceptRequest({required RequestData data}) async {
    print(" ${data.method} Request at ${data.url}");
    print(" On ${data.headers} ${data.body}");
    return data;
  }

  @override
  Future<ResponseData> interceptResponse({required ResponseData data}) async {
    print(" ${data.method} Response from ${data.url}");
    print("${data.headers}");
    print("${data.body}");
    return data;
  }
}
