import 'dart:convert';

import 'package:blusalt_identity_verification/common/exception.dart';
import 'package:blusalt_identity_verification/model/ai_response.dart';
import 'package:blusalt_identity_verification/model/request/nin_by_phone_request.dart';
import 'package:blusalt_identity_verification/network/network_client.dart' as client;
import 'package:http/http.dart';
import 'package:http_interceptor/http_interceptor.dart';

import 'blusalt_sdk.dart';
import 'model/bvn_response.dart';
import 'model/dl_response.dart';
import 'model/ibvn_response.dart';
import 'model/nin_response.dart';
import 'model/nip_response.dart';
import 'model/pvc_response.dart';
import 'model/request/ai.dart';
import 'model/request/bvn_request.dart';
import 'model/request/dl_request.dart';
import 'model/request/ibvn_request.dart';
import 'model/request/nin_request.dart';
import 'model/request/pvc_request.dart';
import 'model/request/nip_request.dart';


var _netWorkClient = client.HttpClient();



Future<BVNResponse> verifyBVN(BVNRequest bvnRequest)  async{
  initializationCheck();

  var response =  await _netWorkClient.executeRequest(client.Request(
      url: "IdentityVerification/BVN",
      body: jsonEncode(bvnRequest),
      apiCredentials: BlusaltSDK.apiCredentials,
      requestMethod: Method.POST));
    var data = response as Response;
    var bvnData = BVNResponse.fromJson(jsonDecode(data.body));
    return bvnData;
  }


Future<NINResponse> verifyNIN(NINRequest ninRequest) async {
  initializationCheck();

  var response = await _netWorkClient.executeRequest(client.Request(
      url: "IdentityVerification/NIN",
      body: jsonEncode(ninRequest),
      apiCredentials: BlusaltSDK.apiCredentials,
      requestMethod: Method.POST));

    var data = response as Response;
    var ninData = NINResponse.fromJson(jsonDecode(data.body));
    return ninData;
}

Future<NINResponse> verifyNINByPhone(NINByPhoneRequest ninRequest) async{
  initializationCheck();

  var response = await _netWorkClient.executeRequest(client.Request(
      url: "IdentityVerification/pNIN",
      body: jsonEncode(ninRequest),
      apiCredentials: BlusaltSDK.apiCredentials,
      requestMethod: Method.POST));
    var data = response as Response;
    var ninData = NINResponse.fromJson(jsonDecode(data.body));
    return ninData;
}

void initializationCheck() {
  if (BlusaltSDK.apiCredentials == null) {
    throw InitializationException(
        "SDK initialization Failed. Please Initialize SDK ");
  }
}

Future<NIPResponse> verifyNIP(NIPRequest request) async {
  initializationCheck();

  var response = await _netWorkClient.executeRequest(client.Request(
      url: "IdentityVerification/NIP",
      body: jsonEncode(request),
      apiCredentials: BlusaltSDK.apiCredentials,
      requestMethod: Method.POST));

    var data = response as Response;
    var nipData = NIPResponse.fromJson(jsonDecode(data.body));
    return nipData;
}

Future<PVCResponse> verifyPVC(PVCRequest request) async {
  initializationCheck();

  var response = await _netWorkClient.executeRequest(client.Request(
      url: "IdentityVerification/PVC",
      body: jsonEncode(request),
      apiCredentials: BlusaltSDK.apiCredentials,
      requestMethod: Method.POST));

    var data = response as Response;
    var pvcData = PVCResponse.fromJson(jsonDecode(data.body));
    return pvcData;
}

Future<DLResponse> verifyDriversLicence(DLRequest request) async{
  initializationCheck();

  var response = await _netWorkClient.executeRequest(client.Request(
      url: "IdentityVerification/DriverLicence",
      body: jsonEncode(request),
      apiCredentials: BlusaltSDK.apiCredentials,
      requestMethod: Method.POST));

    var data = response as Response;
    var dlData = DLResponse.fromJson(jsonDecode(data.body));
    return dlData;
}

Future<IBVNResponse> verifyBVNWithImage(IBVNRequest request) async{
  initializationCheck();

  var response = await _netWorkClient.executeRequest(client.Request(
      url: "IdentityVerification/iBVN",
      body: jsonEncode(request),
      apiCredentials: BlusaltSDK.apiCredentials,
      requestMethod: Method.POST));

    var data = response as Response;
    var ibvnData = IBVNResponse.fromJson(jsonDecode(data.body));
    return ibvnData;
}

Future<FacialVerificationResponse> compareFaces(FacialVerificationRequest request) async{
  initializationCheck();

  var response = await _netWorkClient.executeMultipartRequest(client.MultiPartRequest(
      url: "IdentityVerification/iBVN",
      filenames: [request.image1FilePath, request.image2FilePath],
      apiCredentials: BlusaltSDK.apiCredentials,
      requestMethod: Method.POST));

  var data = response as Response;
  var ibvnData = FacialVerificationResponse.fromJson(jsonDecode(data.body));
  return ibvnData;
}
